@@ -22,6 +22,7 @@ from pyramid.response import Response
 from pyramid.tweens import EXCVIEW
 from string import Template
 
+import hupper
 import logging
 import typing as t
 
@@ -50,10 +51,11 @@ def includeme(config: Configurator) -> None:
     )
 
 
-View = t.Callable[[t.Any, Request], Response]
+Context = t.TypeVar("Context")
+View = t.Callable[[Context, Request], Response]
 
 
-def openapi_view(view: View, info: ViewDeriverInfo) -> t.Optional[View]:
+def openapi_view(view: View, info: ViewDeriverInfo) -> View:
     """View deriver that takes care of request/response validation.
 
     If `openapi=True` is passed to `@view_config`, this decorator will:
@@ -65,7 +67,7 @@ def openapi_view(view: View, info: ViewDeriverInfo) -> t.Optional[View]:
     """
     if info.options.get("openapi"):
 
-        def wrapper_view(context, request):
+        def wrapper_view(context: Context, request: Request) -> Response:
             # Validate request and attach all findings for view to introspect
             request.environ["pyramid_openapi3.validate_response"] = True
             settings = request.registry.settings["pyramid_openapi3"]
@@ -106,10 +108,10 @@ def add_explorer_view(
     :param ui_version: Swagger UI version string
     """
 
-    def register():
+    def register() -> None:
         resolved_template = AssetResolver().resolve(template)
 
-        def explorer_view(request):
+        def explorer_view(request: Request) -> Response:
             settings = config.registry.settings
             if settings.get("pyramid_openapi3") is None:
                 raise ConfigurationError(
@@ -152,6 +154,8 @@ def add_spec_view(
     """
 
     def register() -> None:
+        if hupper.is_active():  # pragma: no cover
+            hupper.get_reloader().watch_files([filepath])
         spec_dict = read_yaml_file(filepath)
 
         validate_spec(spec_dict)
